@@ -1,16 +1,9 @@
-// Copyright (c) 2017 fd developers
-// Licensed under the Apache License, Version 2.0
-// <LICENSE-APACHE or http://www.apache.org/licenses/LICENSE-2.0>
-// or the MIT license <LICENSE-MIT or http://opensource.org/licenses/MIT>,
-// at your option. All files in the project carrying such
-// notice may not be copied, modified, or distributed except
-// according to those terms.
-
 use std::io;
 use std::io::Write;
 use std::process::Command;
 use std::sync::Mutex;
 
+use crate::error::print_error;
 use crate::exit_codes::ExitCode;
 
 /// Executes a command.
@@ -38,11 +31,11 @@ pub fn execute_command(mut cmd: Command, out_perm: &Mutex<()>) -> ExitCode {
             }
         }
         Err(ref why) if why.kind() == io::ErrorKind::NotFound => {
-            print_error!("Command not found: {:?}", cmd);
+            print_error(format!("Command not found: {:?}", cmd));
             ExitCode::GeneralError
         }
         Err(why) => {
-            print_error!("Problem while executing command: {}", why);
+            print_error(format!("Problem while executing command: {}", why));
             ExitCode::GeneralError
         }
     }
