@@ -1,18 +1,13 @@
-// Copyright (c) 2017 fd developers
-// Licensed under the Apache License, Version 2.0
-// <LICENSE-APACHE or http://www.apache.org/licenses/LICENSE-2.0>
-// or the MIT license <LICENSE-MIT or http://opensource.org/licenses/MIT>,
-// at your option. All files in the project carrying such
-// notice may not be copied, modified, or distributed except
-// according to those terms.
-
-use super::CommandTemplate;
-use crate::exit_codes::ExitCode;
-use crate::walk::WorkerResult;
 use std::path::PathBuf;
 use std::sync::mpsc::Receiver;
 use std::sync::{Arc, Mutex};
 
+use crate::error::print_error;
+use crate::exit_codes::{merge_exitcodes, ExitCode};
+use crate::walk::WorkerResult;
+
+use super::CommandTemplate;
+
 /// An event loop that listens for inputs from the `rx` receiver. Each received input will
 /// generate a command with the supplied command template. The generated command will then
 /// be executed, and this process will continue until the receiver's sender has closed.
@@ -21,7 +16,8 @@ pub fn job(
     cmd: Arc<CommandTemplate>,
     out_perm: Arc<Mutex<()>>,
     show_filesystem_errors: bool,
-) {
+) -> ExitCode {
+    let mut results: Vec<ExitCode> = Vec::new();
     loop {
         // Create a lock on the shared receiver for this thread.
         let lock = rx.lock().unwrap();
@@ -32,18 +28,20 @@ pub fn job(
             Ok(WorkerResult::Entry(val)) => val,
             Ok(WorkerResult::Error(err)) => {
                 if show_filesystem_errors {
-                    print_error!("{}", err);
+                    print_error(err.to_string());
                 }
                 continue;
             }
             Err(_) => break,
         };
 
-        // Drop the lock so that other threads can read from the the receiver.
+        // Drop the lock so that other threads can read from the receiver.
         drop(lock);
-        // Generate a command and execute it.
-        cmd.generate_and_execute(&value, Arc::clone(&out_perm));
+        // Generate a command, execute it and store its exit code.
+        results.push(cmd.generate_and_execute(&value, Arc::clone(&out_perm)))
     }
+    // Returns error in case of any error.
+    merge_exitcodes(&results)
 }
 
 pub fn batch(
@@ -55,7 +53,7 @@ pub fn batch(
         WorkerResult::Entry(val) => Some(val),
         WorkerResult::Error(err) => {
             if show_filesystem_errors {
-                print_error!("{}", err);
+                print_error(err.to_string());
             }
             None
         }
