@@ -243,15 +243,23 @@ impl TestEnv {
     /// Assert that calling *fd* with the specified arguments produces the expected error,
     /// and does not succeed.
     pub fn assert_failure_with_error(&self, args: &[&str], expected: &str) {
-        let status = self.assert_error_subdirectory(".", args, expected);
+        let status = self.assert_error_subdirectory(".", args, Some(expected));
         if status.success() {
             panic!("error '{}' did not occur.", expected);
         }
     }
 
+    /// Assert that calling *fd* with the specified arguments does not succeed.
+    pub fn assert_failure(&self, args: &[&str]) {
+        let status = self.assert_error_subdirectory(".", args, None);
+        if status.success() {
+            panic!("Failure did not occur as expected.");
+        }
+    }
+
     /// Assert that calling *fd* with the specified arguments produces the expected error.
     pub fn assert_error(&self, args: &[&str], expected: &str) -> process::ExitStatus {
-        self.assert_error_subdirectory(".", args, expected)
+        self.assert_error_subdirectory(".", args, Some(expected))
     }
 
     /// Assert that calling *fd* in the specified path under the root working directory,
@@ -260,7 +268,7 @@ impl TestEnv {
         &self,
         path: P,
         args: &[&str],
-        expected: &str,
+        expected: Option<&str>,
     ) -> process::ExitStatus {
         // Setup *fd* command.
         let mut cmd = process::Command::new(&self.fd_exe);
@@ -270,17 +278,19 @@ impl TestEnv {
         // Run *fd*.
         let output = cmd.output().expect("fd output");
 
-        // Normalize both expected and actual output.
-        let expected_error = normalize_output(expected, true, self.normalize_line);
-        let actual_err = normalize_output(
-            &String::from_utf8_lossy(&output.stderr),
-            false,
-            self.normalize_line,
-        );
+        if let Some(expected) = expected {
+            // Normalize both expected and actual output.
+            let expected_error = normalize_output(expected, true, self.normalize_line);
+            let actual_err = normalize_output(
+                &String::from_utf8_lossy(&output.stderr),
+                false,
+                self.normalize_line,
+            );
 
-        // Compare actual output to expected output.
-        if !actual_err.trim_start().starts_with(&expected_error) {
-            panic!(format_output_error(args, &expected_error, &actual_err));
+            // Compare actual output to expected output.
+            if !actual_err.trim_start().starts_with(&expected_error) {
+                panic!(format_output_error(args, &expected_error, &actual_err));
+            }
         }
 
         return output.status;
